@@ -1427,11 +1427,24 @@ AccountNumberCheck::Result method_C5(const int *account, int *weight) {
     }
     return AccountNumberCheck::ERROR;
 }
+// method C6 was changed for due-date March, 9 2009
+// checked with Bundesbank-Testnumbers
 AccountNumberCheck::Result method_C6(const int *account, int *weight) {
     number2Array("0121212120", weight);
+    // quick hack: the processed number-prefix boils down to a constant factor:
+    // e.g. for prefix="5499570" -> factor=q(2*5)+q(4)+q(2*9)+q(9)+q(2*5)+q(7)+q(0)=1+4+9+9+1+7+0=31
+    int mod_factor=0;
+    switch(account[0]) {
+	case 0:	mod_factor=30;	break;	// prefix="4451970"
+	case 1:	mod_factor=33;	break;	// prefix="4451981"
+	case 2:	mod_factor=36;	break;	// prefix="4451992"
+	case 7:	mod_factor=31;	break;	// prefix="5499570"
+	case 9:	mod_factor=40;	break;	// prefix="5499579"
+	default:
+	  return AccountNumberCheck::ERROR;
+    }
     int result	= algo03(10, weight, true, account, 1, 8);
-    // quick hack: the processed leading numbers "5499570" boil down to a constant factor of 31
-    result	+= 31;
+    result	+= mod_factor;
     // calc the check number, note: 1000 should be large enough.
     result	= (1000 - result) % 10;
     // compare the result with the real check number
@@ -1462,6 +1475,44 @@ AccountNumberCheck::Result method_C9(const int *account, int *weight) {
     return method_07(account,weight);				// variant 2
 }
 
+// new methods D0 and D1 valid from September 8th, 2008, checked with Bundesbank-Testnumbers
+AccountNumberCheck::Result method_D0(const int *account, int *weight) {
+    if (account[0] != 5 || account[1] != 7)
+	return method_20(account,weight);			// variant 1
+    else
+	return AccountNumberCheck::OK;				// variant 2, "method 9" again :-))
+}
+AccountNumberCheck::Result method_D1(const int *account, int *weight) {
+    if (account[0] == 0 || account[0] == 3 || account[0] == 9)
+	return AccountNumberCheck::ERROR;
+    // note: from here on this method is quite similar to method_C6
+    number2Array("2121212120", weight);
+    int result	= algo03(10, weight, true, account, 0, 8);
+    // quick hack: the processed leading numbers "428259" boil down to a constant factor of 29
+    result	+= 29;
+    // calc the check number, note: 1000 should be large enough.
+    result	= (1000 - result) % 10;
+    // compare the result with the real check number
+    if (account[9] == result)
+	return AccountNumberCheck::OK;
+    else 
+	return AccountNumberCheck::ERROR;
+}
+
+// new methods D2 and D3 valid from December 8th, 2008
+AccountNumberCheck::Result method_D2(const int *account, int *weight) {
+    if (AccountNumberCheck::OK == method_95(account,weight) )	// variant 1
+	return AccountNumberCheck::OK;
+    if (AccountNumberCheck::OK == method_00(account,weight) )	// variant 2
+	return AccountNumberCheck::OK;
+    return method_68(account,weight);				// variant 3
+}
+AccountNumberCheck::Result method_D3(const int *account, int *weight) {
+    if (AccountNumberCheck::OK == method_00(account,weight) )	// variant 1
+	return AccountNumberCheck::OK;
+    return method_27(account,weight);				// variant 2
+}
+
 struct method_func_s {
     const char *str;
     AccountNumberCheck::MethodFunc func;
@@ -1593,6 +1644,10 @@ const struct method_func_s cb_funcs[] = {
   { "C7", method_C7},
   { "C8", method_C8},
   { "C9", method_C9},
+  { "D0", method_D0},
+  { "D1", method_D1},
+  { "D2", method_D2},
+  { "D3", method_D3},
   { 0, 0} // Important: The array has to end with the {0,0} entry,
 	  // otherwise initMethodMap() will runaway and crash!
 };
